/* C++ library */
#include <vector>
#include <iostream>
#include <complex>
#include <cmath>
#include <map>
#include <limits>

/* Fix std::isnan problem in Windows */
#ifndef _isnan
#define _isnan isnan
#endif
